use std::collections::HashMap;
use std::fmt::Arguments;
use chrono::{Datelike, Local, Timelike};
use std::io::{self, Write};

#[derive(Eq, PartialEq, Hash)]
pub enum LogLevel {
    NOLOG,
    INFO,
    CORRECT,
    WARNING,
    ERROR,
}

#[allow(dead_code)]
pub enum ColorFormat {
    BLUE,
    GREEN,
    YELLOW,
    RED,
    LIGHTRED,
    WHITE,
    RESET,
}

impl ColorFormat {
    fn as_str(&self) -> &'static str {
        match self {
            ColorFormat::BLUE => "\x1b[34m",
            ColorFormat::GREEN => "\x1b[32m",
            ColorFormat::YELLOW => "\x1b[33m",
            ColorFormat::RED => "\x1b[31m",
            ColorFormat::LIGHTRED => "\x1b[91m",
            ColorFormat::WHITE => "\x1b[97m",
            ColorFormat::RESET => "\x1b[0m",
        }
    }
}

fn get_color_print() -> HashMap<LogLevel, &'static str> {
    let mut map = HashMap::new();
    map.insert(LogLevel::NOLOG, "");
    map.insert(LogLevel::INFO, concat!("\x1b[34m","[INFO]","\x1b[0m"));
    map.insert(LogLevel::CORRECT, concat!("\x1b[32m","[OK  ]", "\x1b[92m"));
    map.insert(LogLevel::WARNING, concat!("\x1b[33m", "[WARN]", "\x1b[93m"));
    map.insert(LogLevel::ERROR, concat!("\x1b[31m", "[ERR ]", "\x1b[91m"));
    map
}

pub fn log_format(level: LogLevel, args: Arguments) {
    let color_map = get_color_print();
    let header = color_map.get(&level).copied().unwrap_or("");
    let message = format!("{}", args);
    let now = Local::now();
    let time = format!(
        "[{:02}/{:02}/{} {:02}:{:02}:{:02}]",
        now.day(),
        now.month(),
        now.year(),
        now.hour(),
        now.minute(),
        now.second()
    );

    if level == LogLevel::NOLOG {
        println!("{}\t\t{}", time, message);
    } else if level != LogLevel::INFO {
        println!("{}{}\t{}{}", header, time, message, ColorFormat::RESET.as_str());
    } else {
        println!("{}{}\t{}", header, time, message);
    }
    io::stdout().flush().unwrap();
}

#[macro_export]
macro_rules! logf {
    ($level:expr, $fmt:expr $(, $arg:expr)* $(,)?) => {
        log::log_format($level, format_args!($fmt $(, $arg)*))
    };
}